package ch04_lambdas;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

import ch05_javagrundlagen.Person;

/**
 * Beispielprogramm zur Demonstration von komplexeren Bedingungen mit Pr�dikaten.
 * 
 * @author Michael Inden
 * 
 * Copyright 2014, 2017, 2020 by Michael Inden 
 */
public class ComplexPredicatesExample 
{
public static void main(final String[] args)
{
    final List<Person> persons = new ArrayList<>();
    persons.add(new Person("Michael", LocalDate.of(1971, 2, 7), "Zrich"));
    persons.add(new Person("Barbara", LocalDate.of(1973, 3, 24), "Hamburg"));
    persons.add(new Person("Lili", LocalDate.of(1979, 7, 14), "Zrich"));
    persons.add(new Person("Tom", LocalDate.of(2017, 5, 5), "Kiel"));
	
	// Einfache Prdikate formulieren
	final Predicate<Person> isAdult = person -> person.getAge() >= 18;
	final Predicate<Person> livesInKiel = 
	                        person -> person.getCity().equals("Kiel");

	// Negation einzelner Prdikate
	final Predicate<Person> isYoung = isAdult.negate();
	
	// Kombination von Prdikaten mit AND
	final Predicate<Person> boysFromKiel = isYoung.and(livesInKiel);
	
	// Kombination von Prdikaten mit OR
	final Predicate<Person> boysFromKielOrLivingInHamburg = 
	                        boysFromKiel.or(person -> 
	                                        person.getCity().equals("Hamburg"));
	
	persons.removeIf(boysFromKielOrLivingInHamburg);
	System.out.println(persons);
}
}