package ch05_javagrundlagen;

import java.time.LocalDate;
import java.time.Period;
import java.util.Objects;

/**
 * Beispielklasse zur Demonstration der Implementierung von equals()
 * 
 * @author Michael Inden
 * 
 * Copyright 2011, 2014, 2020 by Michael Inden 
 */
public final class Person
{
    private final String name;
    private final LocalDate birthday;
    private final String city;

    public Person(final String name, final LocalDate birthday, final String city)
    {
    	// Hilfsklasse Objects neu in JDK 7
    	Objects.requireNonNull(name, "parameter 'name' must not be null!");
    	Objects.requireNonNull(birthday, "parameter 'birthday' must not be null!");
    	Objects.requireNonNull(city, "parameter 'city' must not be null!");
        
        this.name = name;
        this.birthday = birthday;
        this.city = city;
    }

    @Override
    public String toString()
    {
        return "Person [name=" + name + ", birthday=" + birthday + ", city=" + city + "]";
    }

    public final String getName()       { return name; }
    public final LocalDate getBirthday()   { return birthday; }
    public final String getCity()       { return city; }
   
       
    public boolean equals(Object other)
    {
        if (other == null)          // null safe
            return false;
            
        if (this == other)          // reflexive
            return true;
            
        if (this.getClass() != other.getClass())   // same type ?
            return false;      
        
        final Person otherPerson = (Person)other;
        return compareAttributes(otherPerson);
    }
    
    private boolean compareAttributes(final Person otherPerson)
    {
        return this.getName().equals(otherPerson.getName()) &&                
               this.getBirthday().equals(otherPerson.getBirthday()) &&
               this.getCity().equals(otherPerson.getCity());           
    }      
    
    // Vermeide FindBugs Warning, obwohl diese Klasse niemals in Hash-Container verwendet wird
    @Override
    public int hashCode()
    {
    	return Objects.hash(this.name, this.birthday, this.city);
    } 

    public int getAge()
    {
        return Period.between(birthday, LocalDate.now()).getYears();
    }
    
    public boolean isAdult()
    {
        return getAge() >= 18;
    }
}