package ch05_javagrundlagen.basics;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Objects;

/**
 * Beispielprogramm zur Demonstration von Ausgaben mit toString()
 * <br>
 * Verbesserungen der Stringausgabe fr Arrays
 * 
 * @author Michael Inden
 * 
 * Copyright 2011, 2020 by Michael Inden 
 */
public class ArraysToStringExample
{
    public static void main(final String[] args)
    {
        final Person tom = new Person("Tom", LocalDate.now(), "Hamburg");
        final Person jerry = new Person("Jerry", LocalDate.now(), "Kiel");
        final Object[] persons = new Object[] { tom, jerry };

        System.out.println("Object[]: " + Arrays.asList(persons));   // JDK 1.4
        System.out.println("Object[]: " + Arrays.toString(persons)); // JDK 5.0
    }

    private ArraysToStringExample()
    {
    }

    private static class Person
    {
        private final String name;
        private final LocalDate   birthday;
        private final String city;

        public Person(final String name, final LocalDate birthday, final String city)
        {
        	// Hilfsklasse Objects neu in JDK 7
        	Objects.requireNonNull(name, "parameter 'name' must not be null!");
        	Objects.requireNonNull(city, "parameter 'city' must not be null!");
        	Objects.requireNonNull(city, "parameter 'city' must not be null!");
            
            this.name = name;
            this.birthday = birthday;
            this.city = city;
        }

        public final String getName()
        {
            return name;
        }

        public final LocalDate getBirthday()
        {
            return birthday;
        }

        public final String getCity()
        {
            return city;
        }

        @Override
        public String toString()
        {
            return getClass().getSimpleName() + ": " + 
                   "Name='" + getName() + "' " +		 
                   "Birthday='" + getBirthday() + "' " +  
            	   "City='" + getCity() + "'";                          
        }
    }
}