package ch06_collections.basics;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * Beispielprogramm zur Demonstration des Lschens aus einer Collection whrend einer Iteration.
 * durch Aufruf der Methode remove() des Iterators.
 * Allerdings wird hier bewut ein zweiter Aufruf durchgefhrt ohne zuvor next() aufzurufen.
 * Das fhrt zu einer java.lang.IllegalStateException.
 * 
 * @author Michael Inden
 * 
 * Copyright 2011 by Michael Inden 
 */
public final class IteratorDoubleRemoveExample
{
    private static void removeEntriesWithPrefix(final List<String> entries, final String prefix)
    {
        final Iterator<String> it = entries.iterator();
        while (it.hasNext())
        {
            final String name = it.next();
            if (name.startsWith(prefix))
            {
                it.remove(); // KORREKT: Zugriff ber remove() des Iterators
                it.remove(); // PROBLEM: it.next()-Aufruf fehlt
            }
        }
    }

    public static void main(final String[] args)
    {
        final String[] names = { "Andreas", "Carsten", "Clemens", "Merten", "Michael", "Peter" };

        // JDK 7: Diamon Operator
        final List<String> namesList = new ArrayList<>();
        namesList.addAll(Arrays.asList(names));

        removeEntriesWithPrefix(namesList, "M");
        System.out.println(namesList);
    }
}
