package ch06_collections.jdk8.map;

import static ch06_collections.jdk8.map.DemoData.createDemoData;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Beispielprogramm demonstriert verschiedene Methoden, um die Hufigkeiten von Wrtern in einem Text zu ermitteln 
 * und hier den Nachbau von getOrDefault() zu zeigen
 * 
 * @author Michael Inden
 * 
 * Copyright 2014 by Michael Inden 
 */
public class GetOrDefaultExample 
{
	public static void main(final String[] args)
	{
		final List<String> wordList = createDemoData();
		
		final Map<String, Integer> wordCounts = new TreeMap<>(); 
		for (final String word : wordList) 
		{
			wordCounts.merge(word, 1, Integer::sum); 
		}
				
		System.out.println(getOrDefaultSimplified(wordCounts, "Liste", -1));
		System.out.println(getOrDefaultSimplified(wordCounts, "???", -1));
	}
	
	// Achtung: Nur fr Singlethreading korrekt
	public static <K,V> V getOrDefaultSimplified(final Map<K,V> map,
							 					 final K key, 
	                                             final V defaultValue)
	{
		if (!map.containsKey(key))
			return defaultValue;

		return map.get(key);
	}
}