package ch09_applikationsbausteine.intro;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

public class CsvProcessingExample {

	public static void main(String[] args) {
		
		final List<String> words = Arrays.asList("Text", "is", "concatenated");
		
		concatenate(words);
		
		final List<String> emptyInput = Arrays.asList();
		
		concatenate(emptyInput);
		
		final List<String> words2 = Arrays.asList("Text", null, "concatenated");
		
		concatenate(words2);
	}

	private static void concatenate(final List<String> words) {
		final String concatenated = toCommaSeparatedString(words);
		System.out.println("concatenated: " + concatenated);

		final String concatenatedJdk8 = toCommaSeparatedStringJDK8(words);
		System.out.println("concatenated JDK 8: " + concatenatedJdk8);
	}

	public static String toCommaSeparatedString(final List<String> inputs) 
	{
		if (inputs == null || inputs.isEmpty())
			return "";

		final StringBuilder result = new StringBuilder();
		final Iterator<String> it = inputs.iterator();
		while (it.hasNext())
		{
			final String value = it.next();
			if (value != null)  // skip null
			{
				result.append(value);
				if (it.hasNext())
					result.append(",");		
			}
	 	}
		return result.toString();
	}
	
	public static String toCommaSeparatedStringJDK8(final List<String> inputs) 
	{		
		return inputs.stream().collect(Collectors.joining(","));
	}
	
	public static List<String> fromCommaSeparatedString(final String input) 
	{
		if (input == null || input.isEmpty()) 
			return new ArrayList<>();

		final List<String> result = new ArrayList<>();
		final String[] tokens = input.split( "," );
		for (final String token : tokens)
		{
	  		result.add(token.trim());
		}
		return result;
	}
}
