package ch11_modern_concurrency.completablefuture;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public class CompletableFutureProcessingChainExample
{
    public static void main(final String[] args) throws InterruptedException, 
                                                        ExecutionException
    {
        new CompletableFutureProcessingChainExample().perform();
    }
    
    public void perform() throws InterruptedException, ExecutionException
    {
        // 1) supplyAsync(): eine einzelne Aktion parallel zum Aufrufer ausfhren  
        final CompletableFuture<String> supplyAsync = 
              CompletableFuture.supplyAsync(() -> longRunningCreateMsg(2));
    
        // 2) thenAcceptAsync(): Aktion nach Abschluss der Ausfhrung  
        supplyAsync.thenAcceptAsync(this::notifySubscribers);
    
        // 3) exceptionally(): Exception-Mapping  
        CompletableFuture.supplyAsync(this::failingMsg)
                         .exceptionally(ex -> "FAILED IN BETWEEN")
                         .thenAccept(this::notifySubscribers);    
        
        Thread.sleep(5_000);
    }
    
    public String longRunningCreateMsg(final int durationInSecs)
    {
    	System.out.println(getCurrentThread() + " >>> longRunningCreateMsg");
    	
        sleepInSeconds(durationInSecs);
        System.out.println(getCurrentThread() + " <<< longRunningCreateMsg");
        
        return "longRunningCreateMsg";
    }
    
    public String getCurrentThread()
    {
    	return Thread.currentThread().getName();
    }
    
    public void notifySubscribers(final String msg)
    {
        System.out.println(getCurrentThread() + " notifySubscribers: " + msg);
    }
    
    public String failingMsg()
    {
        throw new IllegalStateException("ISE");
    }
    
    private void sleepInSeconds(final int durationInSeconds)
    {
        try
        {
            TimeUnit.SECONDS.sleep(durationInSeconds);
        }
        catch (InterruptedException e) 
        {  /* not possible here */ }
    }
}

