package ch11_modern_concurrency.executors;

import java.time.LocalTime;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

/**
 * Beispiel fr die parallele Abarbeitung mit einem ExecutorService
 * <br>
 * Zwei Jobs vom Typ CalcDurationInMs mit 5 und 10 Sekunden Wartezeit 
 * werden angestossen und dann wird auf deren Ende gewartet
 * 
 * @author Michael Inden
 * 
 * Copyright 2011, 2014, 2020 by Michael Inden 
 */
public final class ExecutorServiceExample
{
    public static void main(final String[] args) throws InterruptedException
    {
        final int POOL_SIZE = 3;
        final ExecutorService executorService = 
        				      Executors.newFixedThreadPool(POOL_SIZE);

        // Definition und Start zweier Tasks 
        final Future<Long> future1 = executorService.submit(new CalcDurationInMs(TimeUnit.SECONDS, 5));
        final Future<Long> future2 = executorService.submit(new CalcDurationInMs(TimeUnit.SECONDS, 10));

        System.out.println("Start: " + LocalTime.now());
        try
        {
            // synchron auf das Ende von Task 1 warten 
            final Long result1 = future1.get();
            System.out.println("After Job 1: " + LocalTime.now());
            System.out.println(result1);
            // Zugriff nach 5 Sekunden sollte false liefern
            System.out.println("isDone? Job 2: " + future2.isDone());

            // synchron auf das Ende von Task 2 warten
            final Long result2 = future2.get();
            System.out.println("After Job 2: " + LocalTime.now());
            System.out.println(result2);
        }
        catch (final InterruptedException | ExecutionException ex)
        {
            // Kann in diesem Beispiel nicht auftreten, s. o. 
        }

        // Aufruf, um Thread-Pool zu beenden und JVM-Terminierung zu ermglichen 
        executorService.shutdown();
        // executorService.awaitTermination(5, TimeUnit.SECONDS);
    }

    private ExecutorServiceExample()
    {
    }
}
