package ch11_modern_concurrency.forkjoin;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveTask;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class CubicSummerFJ_V2
{
    public static void main(String[] args)
    {
        var values = new ArrayList<Long>();
        // Ab 100 Millionen schneller
        for (int i = 0; i < 100_000_000; i++)
            values.add(i, (long) (Math.random() * 7));

        // performance
        long start = System.currentTimeMillis();
        long cubicSum = 0;
        for (int i = 0; i < values.size(); i++)
        {
            cubicSum += Math.pow(values.get(i), 3);
        }
        long end = System.currentTimeMillis();
        System.out.println("result " + cubicSum + " / loop ms: " + (end - start));
        
        long start2 = System.currentTimeMillis();
        long cubicSum2 = cubicSumWithFj(values);
        long end2 = System.currentTimeMillis();
        System.out.println("result " + cubicSum2 + " / fj ms: " + (end2 - start2));
    }

    private static long cubicSumWithFj(final List<Long> values)
    {
        final ForkJoinPool pool = ForkJoinPool.commonPool();

        return pool.invoke(new CubicSumTask(values));
    }

    public static class CubicSumTask extends RecursiveTask<Long>
    {
        private static final int THRESHOLD = 7;

        private final List<Long> values;

        public CubicSumTask(List<Long> values)
        {
            this.values = values;
        }

        @Override
        protected Long compute()
        {
            if (values.size() < THRESHOLD)
            {
                long sum = 0;
                for (int i = 0; i < values.size(); i++)
                    sum += Math.pow(values.get(i), 3);
                return sum;
            }

            final int midPos = values.size() / 2;
            final List<Long> lowerPart = values.subList(0, midPos);
            final List<Long> upperPart = values.subList(midPos, values.size());

            final CubicSumTask left = new CubicSumTask(lowerPart);
            final CubicSumTask right = new CubicSumTask(upperPart);
            invokeAll(left, right);

            return left.join() + right.join();
        }
    }
}
