package ch11_modern_concurrency.forkjoin;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveTask;
import java.util.concurrent.TimeUnit;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class MergSortWithListAndFJ 
{
	public static void main(final String[] args) 
	{	   
	    var values = randomValues();
	    
	    long start = System.nanoTime();
	    List<Long> sorted = mergesort(values);
	    long end = System.nanoTime();
	    System.out.println("#values: " + sorted.size());
	    System.out.println("mergesort ms: " + TimeUnit.NANOSECONDS.toMillis(end - start));

	    values = randomValues();
	    long start1 = System.nanoTime();
	    sorted = mergesortWithInsertionSort(values);
        long end1 = System.nanoTime();
	    System.out.println("#values: " + sorted.size());
        System.out.println("optimized mergesort ms: " + TimeUnit.NANOSECONDS.toMillis(end1 - start1));
        
        values = randomValues();
	    long start2 = System.nanoTime();
        sorted = mergesortWithFj(values);
	    long end2 = System.nanoTime();
	    System.out.println("#values: " + sorted.size());
	    System.out.println("sort fj ms:" + TimeUnit.NANOSECONDS.toMillis(end2 - start2));
	}

    protected static ArrayList<Long> randomValues()
    {
        var values = new ArrayList<Long>();
	    for (int i = 0; i < 100_000; i++)
	        values.add(i, (long) i);
	    Collections.shuffle(values);
        return values;
    }
	
    private static List<Long> mergesortWithFj(final List<Long> values)
    {
        final ForkJoinPool pool = ForkJoinPool.commonPool();

        return pool.invoke(new MergesortTask(values));
    }

	public static class MergesortTask extends RecursiveTask<List<Long>> 
	{
		private static final int THRESHOLD = 7;
		private final List<Long> values;
	
		public MergesortTask(List<Long> values) {
			this.values = values;
		}
	
		@Override
		protected List<Long> compute() 
		{
			if (values.size() < THRESHOLD) 
			{
				values.sort(Comparator.naturalOrder());
				return values;
			}
	
			final int midPos = values.size() / 2;
			final List<Long> lowerPart = values.subList(0, midPos);
			final List<Long> upperPart = values.subList(midPos, values.size());
			
			final MergesortTask left = new MergesortTask(lowerPart);
			final MergesortTask right = new MergesortTask(upperPart);
			invokeAll(left, right);
	
			return Merger.merge(left.join(), right.join());
		}
	}	


	static List<Long> mergesort(final List<Long> values)
	{
	    if (values.size() <= 1)
	        return values;
	
	    final int midPos = values.size() / 2;
	    final List<Long> lowerPart = values.subList(0, midPos);
	    final List<Long> upperPart = values.subList(midPos, values.size());
	
	    final List<Long> sortedLeft = mergesort(lowerPart);
	    final List<Long> sortedRight = mergesort(upperPart);
	
	    return Merger.merge(sortedLeft, sortedRight);
	}
	
	static List<Long> mergesortWithInsertionSort(final List<Long> values)
	{
	    if (values.size() < 7) 
	    {
	        values.sort(Comparator.naturalOrder());
	        return values;
	    }
	
	    final int midPos = values.size() / 2;
	    final List<Long> lowerPart = values.subList(0, midPos);
	    final List<Long> upperPart = values.subList(midPos, values.size());
	
	    final List<Long> sortedLeft = mergesortWithInsertionSort(lowerPart);
	    final List<Long> sortedRight = mergesortWithInsertionSort(upperPart);
	
	    return Merger.merge(sortedLeft, sortedRight);
	}
}
