package ch11_modern_concurrency.intro;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public class BlockingQueueExample
{
    public static void main(final String[] args) throws Exception
    {
        final BlockingQueue<Integer> queue = new ArrayBlockingQueue<>(4);

        new Thread(new Producer(queue)).start();
        final Thread consumerThread = new Thread(new Consumer(queue));
        consumerThread.start();
    
        Thread.sleep(7000);        
        consumerThread.interrupt();
    }

    static class Producer implements Runnable
    {
        private BlockingQueue<Integer> queue = null;

        public Producer(BlockingQueue<Integer> queue)
        {
            this.queue = queue;
        }

        @Override
		public void run()
        {
            // erste 4 gehen direkt, dann warten auf Consumer
            for (int i = 0; i < 8; i++)
            {
                try
                {
                    System.out.println("putting " + i);
                    queue.put(i);
                }
                catch (Exception e1)
                {
                    e1.printStackTrace();
                }
            }            
            
            // Consumer muss immer 2 Sekunden warten, bis Erzeugung fertig
            try
            {
                queue.put(1111);
                Thread.sleep(2000);
                queue.put(2222);
                Thread.sleep(2000);
                queue.put(3333);
            }
            catch (InterruptedException e)
            {
                e.printStackTrace();
            }
        }
    }

    static class Consumer implements Runnable
    {
        private BlockingQueue<Integer> queue;

        public Consumer(BlockingQueue<Integer> queue)
        {
            this.queue = queue;
        }

        @Override
		public void run()
        {
            while (!Thread.currentThread().isInterrupted())
            {
                try
                {
                    System.out.println("Consuming " + queue.take());
                    Thread.sleep(100);
                }
                catch (InterruptedException e)
                {
                    Thread.currentThread().interrupt();
                }
            }
        }
    }
}
