package ch12_advancedjava.http2;

import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.concurrent.CompletableFuture;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public class Http2AsyncExample
{
    public static void main(final String[] args) throws Exception
    {
        var uri = new URI("https://www.oracle.com/index.html");

        var request = HttpRequest.newBuilder(uri).GET().build();
        var asString = HttpResponse.BodyHandlers.ofString();

        var httpClient = HttpClient.newHttpClient();
        final CompletableFuture<HttpResponse<String>> asyncResponse = httpClient.sendAsync(request, asString);
                                                      
        // Variante: Verarbeitung, sobald die Response eintrifft
        // asyncResponse.thenAccept(response -> printResponseInfo(response));
        
        // Variante: Verarbeitung mit eigenem Warten, Abfrage auf Erfolg und
        // bei nicht erfolgreichem Abschluss mit einem Abbruch per cancel(true)
        waitForCompletion(asyncResponse);
        if (asyncResponse.isDone())
        {
            final HttpResponse<String> response = asyncResponse.get();
            printResponseInfo(response);
        }
        else
        {
            asyncResponse.cancel(true);
            System.err.println("timeout");
        }
    }
    
    private static void waitForCompletion(final CompletableFuture<HttpResponse<String>> 
                                          asyncResponse) throws InterruptedException
    {
        int i = 0;
        while (!asyncResponse.isDone() && i < 10)
        {
            System.out.println("Step " + i);
            i++;
            Thread.sleep(200);
        }
    }
    
    private static void printResponseInfo(final HttpResponse<String> response)
    {
        var responseCode = response.statusCode();
        var responseBody = response.body();
    
        System.out.println("Status: " + responseCode);
        System.out.println("Body:   " + responseBody);
    }
}

