package ch12_advancedjava.http2;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse.BodyHandlers;
import java.time.Duration;
import java.time.LocalDate;
import java.time.Month;
import java.time.Period;
import java.time.temporal.TemporalAdjusters;
import java.util.Optional;
import java.util.function.Consumer;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public class RealWorldRestCallExample 
{
    public static void main(String[] args) throws IOException, InterruptedException
    {
    	// HTTP-2-API + Date and Time API 
        HttpClient httpClient = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(5)).build();
    
        // Date and Time API => Stream mit monatlicher Schrittweite 
        LocalDate startDay = LocalDate.of(2020, Month.FEBRUARY, 1);
        LocalDate endDay = LocalDate.of(2020, Month.JUNE, 1);
        Period stepOneMonth = Period.ofMonths(1);
    
        // DATESUNTIL
        startDay.datesUntil(endDay, stepOneMonth).forEach(localDate -> {
    
            // TEMPORALADJUSTERS
            var endOfMonth = localDate.with(TemporalAdjusters.lastDayOfMonth());
    
            // var seit Java 10 und Aufruf mit HTTP/2-API
            var optResponseBody = performGet(httpClient, endOfMonth);
    
	        Runnable noDataInfo = () -> System.out.println("No data for " + endOfMonth);
			Consumer<String> dataNotification = 
					          value -> System.out.println(endOfMonth + 
					        		                      " reported " + value);
					          
	        // OPTIONAL-Handling 			          
			optResponseBody.ifPresentOrElse(dataNotification,
	                                        noDataInfo);
	    });
	}
    
    private static Optional<String> performGet(final HttpClient httpClient, 
                                               final LocalDate desiredDate)
    {
        try
        {
            URI url = URI.create("https://api.exchangeratesapi.io/" + 
                                 desiredDate + "?symbols=CHF");
            
    		HttpRequest httpRequest = 
    		            HttpRequest.newBuilder().GET().uri(url).build();
    
            var response = httpClient.send(httpRequest, BodyHandlers.ofString());
            String respnseBody = response.body();
    
            return Optional.of(respnseBody);
        }
        catch (Exception ex)
        {
            // in diesem Beispiel ausnahmsweise ignorieren 
        }
        return Optional.empty();
    }
}

