package ch12_advancedjava.reflection;

import java.lang.reflect.Array;
import java.util.Arrays;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public class GenericArrayCreationAndConcatExample
{
	public static void main(final String[] args)
	{
	    final String[] arr1 = { "Tim1", "Tom1", "Mike1" }; 
	    final String[] arr2 = { "Tim2", "Tom2", "Mike2" }; 
	    final int[][] values1 = { { 1, 2 }, { 3, 4 } }; 
	    final int[][] values2 = { { 5, 6 }, { 7, 8 } }; 
	    
	    // type safe concatenate (=> using correct type, not Object[]) 
	    final String[] result1 = concat(arr1, arr2); 
	    final int[][] result2 = concat(values1, values2);            
	
	    System.out.println(result1.getClass() + ": " + Arrays.toString(result1)); 
	    System.out.println(result2.getClass() + ": " + Arrays.deepToString(result2)); 
	}
	
    // T[] kann auch mehrdimensional sein 
    public static <T> T[] concat(final T[] first, final T[] second)
    {
        final int len1 = first.length;
        final int len2 = second.length;
    
        final Class<?> componentType = first.getClass().getComponentType();
    
        @SuppressWarnings("unchecked")
        final T[] result = (T[]) createTypedArray(componentType, len1 + len2);
        System.arraycopy(first, 0, result, 0, len1);
        System.arraycopy(second, 0, result, len1, len2);
    
        return result;
    }
    
    @SuppressWarnings("unchecked")
    public static <T> T[] createTypedArray(final Class<T> clazz, final int size) 
    {
        return (T[]) Array.newInstance(clazz, size); 
    }
}
