package ch12_advancedjava.serializable;

import java.awt.Color;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.time.LocalDate;
import java.util.Objects;

import ch05_javagrundlagen.basics.EqualsUtils;

/**
 * Beispielklasse zur Modellierung einer Person
 * <br>
 * Demonstration einer Versionsverwaltung ber die serialVersionUID
 *  
 * @author Michael Inden
 * 
 * Copyright 2011, 2019 by Michael Inden 
 */
public final class PersonWithEyeColorV2 implements Serializable
{
    private String name;

    private String city;

    private LocalDate birthday;

    private Color  eyeColor;

    public PersonWithEyeColorV2(final String name, final String city, final LocalDate birthday, final Color eyeColor)
    {
        this.name = name;
        this.city = city;
        this.birthday = birthday;
        this.eyeColor = eyeColor;
    }

    @Override
    public String toString()
    {
        return super.toString() + "\n" + "[name=" + name + ", city=" + city + ", birthday=" + birthday + ", eyeColor="
               + eyeColor + "]";
    }

    // Achtung: falche Lsung
    private void writeObject(final ObjectOutputStream outStream) throws IOException
    {
        outStream.defaultWriteObject();

        outStream.writeUTF(birthday.toString());

        outStream.writeInt(eyeColor.getRed());
        outStream.writeInt(eyeColor.getGreen());
        outStream.writeInt(eyeColor.getBlue());
    }

    private void readObject(final ObjectInputStream inStream) throws IOException, ClassNotFoundException
    {
        inStream.defaultReadObject();

        final String dateAsString = inStream.readUTF();
        birthday = LocalDate.parse(dateAsString);

        final int red = inStream.readInt();
        final int green = inStream.readInt();
        final int blue = inStream.readInt();
        eyeColor = new Color(red, green, blue);
    }

    // ...

    public final String getName()
    {
        return name;
    }

    public final String getCity()
    {
        return city;
    }

    public final LocalDate getBirthDay()
    {
        return birthday;
    }

    public final Color getEyeColor()
    {
        return eyeColor;
    }

    public boolean equals(final Object other)
    {
        if (other == null) // null safe
            return false;

        if (this == other) // reflexive
            return true;

        if (this.getClass() != other.getClass())
            return false;

        final PersonWithEyeColorV2 otherPerson = (PersonWithEyeColorV2) other;
        return equalsImpl(otherPerson);
    }

    private boolean equalsImpl(final PersonWithEyeColorV2 otherPerson)
    {
        return EqualsUtils.nullSafeEquals(this.getName(), otherPerson.getName())
               && EqualsUtils.nullSafeEquals(this.getCity(), otherPerson.getCity())
               && EqualsUtils.nullSafeEquals(this.getBirthDay(), otherPerson.getBirthDay())
               && EqualsUtils.nullSafeEquals(this.getEyeColor(), otherPerson.getEyeColor());
    }
    
    // Vermeide FindBugs Warning, obwohl diese Klasse niemals in Hash-Container verwendet wird
    @Override
    public int hashCode()
    {
    	return Objects.hash(this.name, this.birthday, this.city);
    } 
}