package ch14_java12_15;

import java.text.CompactNumberFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

/**
 * Beispielklasse fr das Buch "Der Weg zum Java-Profi"
 * 
 * @author Michael Inden
 * 
 *         Copyright 2020 by Michael Inden
 */
public class CustomCompactNumberFormat {
	public static void main(String[] args) {

		// Based on number of array elements, these values for the range 100 to 1014.
		final String[] compactPatterns = { "", "", "", "0 [KB]", "00 [KB]", "000 [KB]", "0 [MB]", "00 [MB]", "000 [MB]",
				"0 [GB]", "00 [GB]", "000 [GB]", "0 [TB]", "00 [TB]", "000 [TB]" };

		final DecimalFormat decimalFormat = (DecimalFormat) NumberFormat.getNumberInstance(Locale.GERMANY);

		final CompactNumberFormat customCompactNumberFormat = new CompactNumberFormat(decimalFormat.toPattern(),
				decimalFormat.getDecimalFormatSymbols(), compactPatterns);

		System.out.println(customCompactNumberFormat.format(990L)); // 990
		System.out.println(customCompactNumberFormat.format(999L)); // 999

		System.out.println(customCompactNumberFormat.format(1_499L)); // 1k
		System.out.println(customCompactNumberFormat.format(1_500L)); // 2k
		System.out.println(customCompactNumberFormat.format(1_999L)); // 2k

		System.out.println(customCompactNumberFormat.format(1_499_999)); // 1m
		System.out.println(customCompactNumberFormat.format(1_500_000)); // 2m
		System.out.println(customCompactNumberFormat.format(1_567_890)); // 2m

		customCompactNumberFormat.setMinimumFractionDigits(1);
		System.out.println(customCompactNumberFormat.format(990L)); // 990
		System.out.println(customCompactNumberFormat.format(999L)); // 999

		System.out.println(customCompactNumberFormat.format(1_499L)); // 1k
		System.out.println(customCompactNumberFormat.format(1_500L)); // 2k
		System.out.println(customCompactNumberFormat.format(1_999L)); // 2k

		System.out.println(customCompactNumberFormat.format(1_499_999)); // 1m
		System.out.println(customCompactNumberFormat.format(1_500_000)); // 2m
		System.out.println(customCompactNumberFormat.format(1_567_890)); // 2m

		// Irritating
		// Locale.US, NumberFormat.Style.SHORT)
		NumberFormat fmt = NumberFormat.getCompactNumberInstance(Locale.US, NumberFormat.Style.SHORT);
		fmt.setMinimumFractionDigits(3);
		System.out.println(fmt.format(10000));
		System.out.println(fmt.format(10012));
		System.out.println(fmt.format(100201));
		System.out.println(fmt.format(1111111));
		/*
		 * US 10.000K 10.012K 100.201K 1.111M
		 */
		/*
		 * GERMANY 10.000 10.012 100.201 1,111 Mio.
		 */
		final DecimalFormat currencyFormat = (DecimalFormat) NumberFormat.getCurrencyInstance();
		final NumberFormat cnf = CompactNumberFormat.getCompactNumberInstance(Locale.GERMAN, NumberFormat.Style.SHORT);
		cnf.setMinimumFractionDigits(2);
		System.out.println(currencyFormat.format(1234.455));
		System.out.println(currencyFormat.format(1234.445));
		System.out.println(currencyFormat.format(12345));
		System.out.println(currencyFormat.format(1234567));
		System.out.println(cnf.format(1234.455));
		System.out.println(cnf.format(1234.445));
		System.out.println(cnf.format(12345));
		System.out.println(cnf.format(1234567));
	}
}
