package ch14_java12_15;

import java.awt.Color;
import java.util.List;
import java.util.Map;

/**
 * Beispielklasse fr das Buch "Der Weg zum Java-Profi"
 * 
 * @author Michael Inden
 * 
 *         Copyright 2020 by Michael Inden
 */
public class RecordsWithGenericsExample {
	record Pair<T1, T2> (T1 first, T2 second) {
	}

	record MultiTypes<K, V, T> (Map<K, V> mapping, T info) {
		public void printTypes() {
			System.out.println("mapping type: " + mapping.getClass());
			System.out.println("info type: " + info.getClass());
		}
	}

	record ListRestrictions<T> (List<T> values, int maxSize) {
		public ListRestrictions {
			if (values.size() > maxSize)
				throw new IllegalArgumentException(
						"too many entries! got: " + values.size() + ", but restricted to " + maxSize);
		}
	}

	public static void main(final String[] args) {
		// Paare mit teilweise verschiedene Typisierungen
		pairExamples();

		//
		multitypesExamples();

		// Prüfe die Restriktion auf n Elemente (hier 5)
		listRestrictionsExamples();
	}

	protected static void pairExamples() {
		var pair1 = new Pair<>("INFO", 4711);
		var pair1b = new Pair<>("INFO", 4711);
		var pair2 = new Pair<>("FIRST", "SECOND");
		var pair3 = new Pair<>("BLUE", Color.BLUE);
		System.out.println(pair1);
		System.out.println(pair2);
		System.out.println(pair3);
		System.out.println(pair1.equals(pair1b));
		System.out.println();
	}

	protected static void multitypesExamples() {
		var mapping = Map.of("tim", 42, "tom", 77, "mike", 49);
		var multiGenericTypes = new MultiTypes<>(mapping, "INFO");
		System.out.println(multiGenericTypes);
		System.out.println("mapping type: " + multiGenericTypes.mapping.getClass());
		System.out.println("info type: " + multiGenericTypes.info.getClass());
		System.out.println();

		var mapping2 = Map.of("tim", "42", "tom", "77", "mike", "49");
		var multiGenericTypes2 = new MultiTypes<>(mapping2, mapping);
		System.out.println(multiGenericTypes2);
		System.out.println("mapping type: " + multiGenericTypes2.mapping.getClass());
		System.out.println("info type: " + multiGenericTypes2.info.getClass());
		System.out.println();
	}

	protected static void listRestrictionsExamples() {
		var timAndMike = List.of("Tim", "Mike");
		var initialRestricted = new ListRestrictions<>(timAndMike, 5);
		System.out.println("values: " + initialRestricted.values());

		var names = List.of("Tim", "Tom", "Peter", "Franz", "Fritz", "Mike");
		var restrictionViolated = new ListRestrictions<>(names, 5);
	}
}
