package ch14_java12_15;

/**
 * Beispielklasse fr das Buch "Der Weg zum Java-Profi"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class SealedTypesExamples
{
    public String UNUSED = "??";
    
    // Eine sealed Klasse muss Subklassen haben
    sealed interface MathOp
        permits BaseOp, Add, Sub
    {
        int calc(int x, int y);
    }

    // Mit non-sealed kann man innerhalb der Vererbungshierarchie
    // Baissklassen bereitstellen
    non-sealed class BaseOp implements MathOp
    {
        @Override
        public int calc(int x, int y) {
            return 0;
        }
    }

    // direkte Implementierung muss final sein
    final class Add implements MathOp
    {
        @Override
        public int calc(int x, int y) {
            return x + y;
        }
    }

    final class Sub implements MathOp
    {
        @Override
        public int calc(int x, int y) {
            return x - y;
        }
    }

    // Ableitung aus baisklasse muss final sein
    final class Mult extends BaseOp
    {
    }

    final class Div extends BaseOp
    {
    }

    // ------------------------------

    // SpecialMathOp is not allowed in the sealed hierarchy
    // final class SpecialMathOp implements MathOp
    // {
    //     @Override
    //     public int calc(int x, int y)
    //     {
    //         return 42;
    //     }
    // }

    // UnsealedMathOp is not allowed in the sealed hierarchy
    // public non-sealed class UnsealedMathOp implements MathOp
    // {
    // }
}
