package ch22_optimierungen.cpubound;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * Beispielprogramm zur Demonstration der Optimierung mithilfe von Memoization
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class FibonacciOptimizedExample
{
	public static void main(final String[] args)
	{
	    for (final int currentValue : new int[] { 10, 30, 40, 45, 50, 70 })
	    {
	        System.out.println("currentValue: " + currentValue);
	        final long start = System.nanoTime();
	        final long fibRecursive = fibonacciOptimized(currentValue);
	        final long end = System.nanoTime();
	
	        System.out.println("fibonacciOptimized took: " + 
	                           TimeUnit.NANOSECONDS.toMillis(end - start) + " ms");
	        System.out.println("fibonacciOptimized is: " + fibRecursive);
	        System.out.println("-----------------------------------------");
	    }
	}

    static long fibonacciOptimized(final int n)
    {
        return fibonacciMemo(n, new HashMap<>());
    }

    static long fibonacciMemo(final int n, final Map<Integer, Long> lookupMap)
    {
        // MEMOIZATION: prfe, ob vorberechnetes Ergebnis existiert 
        if (lookupMap.containsKey(n))
            return lookupMap.get(n);

        // normaler Algorithmus mit Hilfsvariable fr Resultat  
        long result = 0;
        // rekursiver Abbruch 
        if (n == 0 || n == 1)
            result = n;
        // rekursiver Abstieg 
        else
            result = fibonacciMemo(n - 1, lookupMap) + fibonacciMemo(n - 2, lookupMap);

        // MEMOIZATION: speichere berechnetes Ergebnis 
        lookupMap.put(n, result);
        return result;
    }
}