package ch22_optimierungen.memorybound;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import dto.BadVO;
import dto.BiggerBadVO;
import dto.BiggerGoodVO;
import dto.GoodVO;


/**
 * Beispielprogramm fr den Einfluss von primitiven Attributen auf die Gre der
 * serialisierten Daten, die sich aus der eigentlichen Gre der Attribute sowie
 * der zustzlichen Informationen (Klassennamen usw.) fr die Serialisierung ergeben
 * 
 * @author Michael Inden
 * 
 * Copyright 2011, 2019, 2020 by Michael Inden 
 */
public final class VOSerializationSizeExample
{
    public static void main(final String[] args) throws Exception
    {
        final BadVO badVO = new BadVO();
        final File badVOFile = new File("BadVo.ser");
        writeObject(badVO, badVOFile);
        System.out.println("File '" + badVOFile.getName() + "' / size = " + badVOFile.length());
        
        final GoodVO goodVO = new GoodVO();
        final File goodVOFile = new File("GoodVO.ser");
        writeObject(goodVO, goodVOFile);
        System.out.println("File '" + goodVOFile.getName() + "' / size = " + goodVOFile.length());
        
        final BiggerBadVO biggerBadVO = new BiggerBadVO();
        final File biggerBadVOFile = new File("BiggerBadVO.ser");
        writeObject(biggerBadVO, biggerBadVOFile);
        System.out.println("File '" + biggerBadVOFile.getName() + "' / size = " + biggerBadVOFile.length());
        
        final BiggerGoodVO biggerGoodVO = new BiggerGoodVO();
        final File biggerGoodVOFile = new File("BiggerGoodVO.ser");
        writeObject(biggerGoodVO, biggerGoodVOFile);
        System.out.println("File '" + biggerGoodVOFile.getName() + "' / size = " + biggerGoodVOFile.length());
    }

    private static void writeObject(final Object objToWrite, final File file) throws IOException, FileNotFoundException
    {
        try (final ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file)))
        {
            oos.writeObject(objToWrite);
            oos.flush();
        }          
    }             
}
