package ch20_unittests.basics;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertIterableEquals;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.junit.jupiter.api.Test;

/**
 * Besipiel eines einfachen Unit Tests mit JUnit 5
 * 
 * @author Michael Inden
 * 
 * Copyright 2011, 2014, 2019 by Michael Inden 
 */
class AssertionExampleTest 
{
	@Test
	void assertSameContent() 
	{
	    final List<String> result = List.of("Mike", "Peter", "Tim");
	
	    final List<String> expected = Arrays.asList("Mike", "Peter", "Tim");
	    
	    assertEquals(expected, result);
	}

	@Test
	void assertSameContent_ChangedOrder() 
	{
	    final List<String> result = List.of("Peter", "Tim", "Mike");

	    final List<String> expected = List.of("Mike", "Peter", "Tim");
	    
	    assertEquals(expected, result);
	}

	@Test
	void assertSameContent_ChangedOrder_IterableEquals() 
	{
	    final Set<String> result = new TreeSet<>(List.of("Peter", "Tim", "Mike"));
	
	    final List<String> expected  = List.of("Mike", "Peter", "Tim");
	    
	    assertIterableEquals(expected, result);
	}	
	
	@Test
	void assertSameContent_ChangedOrder_IterableEquals_RealWorld() 
	{
	    final Map<String, Integer> result = Map.of("Mike", 49, "Peter", 17, "Tim", 49);
	    final Set<String> keys = new TreeSet<>(result.keySet());
	
	    final List<String> expected  = List.of("Mike", "Peter", "Tim");
	    
	    assertIterableEquals(expected, keys);
	}
}
