package ch20_unittests.basics;

import static ch20_unittests.basics.DiscountCalculatorV3.BIG_DISCOUNT;
import static ch20_unittests.basics.DiscountCalculatorV3.MEDIUM_DISCOUNT;
import static ch20_unittests.basics.DiscountCalculatorV3.NO_DISCOUNT;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.Map;

import org.junit.jupiter.api.Test;

/**
 * Unit-Tests zu der Utility-Klasse DiscountCalculatorV3.
 * 
 * @author Michael Inden
 * 
 * Copyright 2014, 2017 by Michael Inden 
 */
public class DiscountCalculatorV3Test 
{
	private final DiscountCalculatorV3 calculator = new DiscountCalculatorV3();
	
	// Java 9: Collection Factory Methodes
	private static final Map<Integer, String> discountToInfoMap = Map.of(
			NO_DISCOUNT, "no discount",
			MEDIUM_DISCOUNT, "4 % discount",
			BIG_DISCOUNT, "7 % discount"
		);
	
	
	@Test
	void testCalcDiscount_SmallOrder_NoDiscount() 
	{
		final int smallAmount = 20;
		assertDiscount(NO_DISCOUNT, smallAmount);
	}

	@Test
	void testCalcDiscount_MediumOrder_MediumDiscount() 
	{
		final int mediumAmount = 200;
		assertDiscount(MEDIUM_DISCOUNT, mediumAmount);
	}

	@Test
	void testCalcDiscount_BigOrderBigDiscount() 
	{
		final int bigAmount = 2000;
		assertDiscount(BIG_DISCOUNT, bigAmount);
	}
	
	private void assertDiscount(int expected, final int amount) 
	{
		// Java 10: var
		var infoMsg = discountToInfoMap.getOrDefault(expected, "n/a");
		
		assertEquals(expected, calculator.calcDiscount(amount), infoMsg);
	}
	
	// Randflle prfen
	
	@Test
	void testCalcDiscount_Input_49_ShouldHaveNoDiscount() 
	{
		assertDiscount(NO_DISCOUNT, 49);
	}
	
	@Test
	void testCalcDiscount_Input_50_ShouldHaveMediumDiscount() 
	{
		assertDiscount(MEDIUM_DISCOUNT, 50);
	}
	
	@Test
	void testCalcDiscount_Input_51_Should_HaveMediumDiscount() 
	{
		assertDiscount(MEDIUM_DISCOUNT, 51);
	}

	@Test
	void testCalcDiscount_Input_999_ShouldHaveMediumDiscount() 
	{
		assertDiscount(MEDIUM_DISCOUNT, 999);
	}

	@Test
	void testCalcDiscount_Input_1000_ShouldHaveMediumDiscount() 
	{
		assertDiscount(MEDIUM_DISCOUNT, 1000);
	}
	
	@Test
	public void testCalcDiscount_Input_1001_ShouldHaveBigDiscount() 
	{
		assertDiscount(BIG_DISCOUNT, 1001);
	}

	// Ungltige Werte prfen
	
	@Test
	void testCalcDiscount_NegativeInput_ShouldThrowException() 
	{
		assertThrows(IllegalArgumentException.class, 
		              ()-> calculator.calcDiscount(-1));
	}
}
