package ch20_unittests.basics;


import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.function.Executable;

/**
 * Unit-Tests zu der Utility-Klasse MyClass.
 * Dienen zur Demonstration der Benennung von Methodennamen
 * 
 * @author Michael Inden
 * 
 * Copyright 2014, 2017, 2019, 2020 by Michael Inden
 */
class SumWithOffsetCalculatorTest 
{
	@Test
	void testCalcSum_NormalInputs() 
	{
		final int offset = 4711;
		var objectToTest = createSumWithOffsetCalculatorObjectForTest(offset);
		
		final List<Integer> values = Arrays.asList(1, 2, 3, -4);
		final long result = objectToTest.sum_with_offset(values);

		final long expected = 2 + offset;
		assertEquals(expected, result, "sum should be calculated correnty");		
	}

	@Test
	void testCalcSum_SpecialCase_NoValuesGiven() 
	{
		final int offset = 0;
		var objectToTest = createSumWithOffsetCalculatorObjectForTest(offset);

		final long result = objectToTest.sum_with_offset(List.of());

		final long expected = offset;
		assertEquals(expected, result, "sum should be zero for empty list");
	}

	@Test
	void testCalcSum_When_InputIsNull_ThenThrowException() 
	{
		var objectToTest = createSumWithOffsetCalculatorObjectForTest(4711);
		
		final Executable executable = () -> objectToTest.sum_with_offset(null);
		
		assertThrows(NullPointerException.class, executable);
	}

	@Test
	void testGetOffset()
	{
		var objectToTest = createSumWithOffsetCalculatorObjectForTest(42);

		final int result = objectToTest.getOffset();

		final int expected = 42;
		assertEquals(expected, result);
	}

	private SumWithOffsetCalculator 
	        createSumWithOffsetCalculatorObjectForTest(int offset)
	{
		return new SumWithOffsetCalculator(offset);
	}
}