package ch20_unittests.basics;


import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.function.Executable;

/**
 * Unit-Tests zu der Utility-Klasse MyClass.
 * Demonstration des Einsatzes einer Testfixture
 * 
 * @author Michael Inden
 * 
 * Copyright 2014, 2017, 2019, 2020 by Michael Inden
 */
class SumWithOffsetCalculatorWithTestFixtureTest 
{
	// Testfixture
	private int offset;
	private SumWithOffsetCalculator objectToTest;

	@BeforeEach
	void initTestFixture() 
	{
		offset = 4711;
		objectToTest = new SumWithOffsetCalculator(offset);
	}
		
	@Test
	void testGetValue()
	{
		final int result = objectToTest.getOffset();

		final int expected = offset;
		assertEquals(expected, result);
	}
	
	@Test
	void testCalcSum_NormalInputs() 
	{
		final List<Integer> values = Arrays.asList(1, 2, 3, -4);
		final long result = objectToTest.sum_with_offset(values);

		final long expected = 2 + offset;
		assertEquals(expected, result, "sum should be calculated correnty");		
	}

	@Test
	void testCalcSum_When_NoValuesGiven() 
	{
		final long result = objectToTest.sum_with_offset(List.of());

		final long expected = offset;
		assertEquals(expected, result, "sum should be null for empty list");
	}

	@Test
	void testCalcSum_When_InputIsNull_ThenThrowException() 
	{
		final Executable executable = () -> objectToTest.sum_with_offset(null);
		
		assertThrows(NullPointerException.class, executable);
	}
}