package ch20_unittests.ch20_3_misc;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.io.IOException;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.function.Executable;

/**
 * Unit-Tests zur Demonstration des Exception-Handlings
 * 
 * @author Michael Inden
 * 
 * Copyright 2019, 2020 by Michael Inden 
 */
public class AssertThrowsTest 
{	
	@Test
	void arrayIndexOutOfBoundsExceptionExpected() 
	{	
		var numbers = new int[] {1,2,3,4,5,6,7}; 
		
		final Executable action = () ->
		{
			numbers[1_000] = 13;
		};
		
		assertThrows(ArrayIndexOutOfBoundsException.class, action);
	}
	
	@Test
	void illegalStateExceptionExpected() 
	{	
		final Executable action = () ->
		{
			throw new IllegalStateException("XYZ is not initialized");	
		};
		
		assertThrows(IllegalStateException.class, action);
	}
	
	@Test
	void illegalStateExceptionWithMessageTextExpected() 
	{	
		final Executable action = () ->
		{
			throw new IllegalStateException("XYZ is not initialized", 
					                        new IOException("IO"));	
			
		};
		
		final IllegalStateException exception = 
				                    assertThrows(IllegalStateException.class, 
				                    		     action);		
		assertEquals("XYZ is not initialized", exception.getMessage());
		assertEquals(IOException.class, exception.getCause().getClass());
	}
}
