package ch20_unittests.ch20_3_misc;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTimeout;
import static org.junit.jupiter.api.Assertions.assertTimeoutPreemptively;

import java.time.Duration;
import java.util.concurrent.TimeUnit;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.api.function.ThrowingSupplier;

/**
 * Unit-Tests zur Demonstration der JUnit Rule Timeout
 * 
 * @author Michael Inden
 * 
 * Copyright 2019, 2020 by Michael Inden 
 */
public class AssertTimeoutTest
{
    @Test
    void timeout_finshed_before()
    {
        final Executable action = () -> shortRunningAction();

        assertTimeout(Duration.ofMillis(100), action);
    }

    @Test
    void timeoutExceeded_Keep_On_Running()
    {
        final Executable action = () -> longRunningAction();

        // wartet, bis die Aktion fertig ist
        assertTimeout(Duration.ofMillis(500), action);
    }

    @Test
    void timeoutExceeded_Preemptive_Termination()
    {
        final Executable actioin = () -> longRunningAction();

        // unterbricht nach angegebener Time-out-Zeit
        assertTimeoutPreemptively(Duration.ofMillis(500), actioin);
    }

    void longRunningAction() throws InterruptedException
    {
        TimeUnit.SECONDS.sleep(2);
    }

    void shortRunningAction() throws InterruptedException
    {
        TimeUnit.MILLISECONDS.sleep(50);
    }

    @Test
    public void loopForever() throws InterruptedException
    {
        final Executable action = () -> {
            for (;;)
            {
                TimeUnit.SECONDS.sleep(1);
            }
        };

        // ACHTUNG: ganz wichtig, hier assertTimeoutPreemptively()
        assertTimeoutPreemptively(Duration.ofMillis(500), action);
    }

    @Test
    void timeoutExceeded_ComplexCalc()
    {
        final ThrowingSupplier<String> action = () -> complexCalc(50);

        String result = assertTimeout(Duration.ofMillis(500), action);
        assertEquals("EXPECTED_RESULT", result);
    }

    private String complexCalc(int i)
    {
        try
        {
            Thread.sleep(2_000);
        }
        catch (InterruptedException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        return "EXPECTED_RESULT";
    }

}
