package ch20_unittests.ch20_4_parameterized_tests;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.time.LocalDate;
import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.EmptySource;
import org.junit.jupiter.params.provider.MethodSource;
import org.junit.jupiter.params.provider.NullSource;

/**
 * Besipiel fr das Buch "Der Weg zum Java-Profi 5"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class CornerCasesTest
{
    @ParameterizedTest(name = "test ''{0}'' for null or empty")
    @NullSource
    @EmptySource
    void nullAndEmptyStrings(String str)
    {
        assertTrue(str == null || str.isEmpty());
    }

    @ParameterizedTest(name = "test ''{0}'' for null or empty")
    @NullSource
    @EmptySource
    void nullAndEmptyLists(List<?> list)
    {
        assertTrue(list == null || list.isEmpty());
    }

    static class DateRangeDto
    {
        // one of both can be null
        LocalDate from;

        LocalDate to;

        public DateRangeDto(LocalDate from, LocalDate to)
        {
            if (from == null && to == null)
                throw new IllegalArgumentException("invalid range, two nulls");

            this.from = from;
            this.to = to;
        }
    }

    //one of both can be null
    record DateRange(LocalDate from, LocalDate to)
    {
        public DateRange
        {
            if (from == null && to == null)
                throw new IllegalArgumentException("invalid range, two nulls");
        }
    }

    @ParameterizedTest(name = "validateDateRange({0} => {1}")
    @MethodSource("allCombinations")
    void validateDateRangeConstruction(LocalDate from, LocalDate to, Class<? extends Throwable> expectedException)
    {
        if (expectedException != null)
            assertThrows(expectedException, () -> new DateRange(from, to));
        else
            assertNotNull(new DateRange(from, to));
    }

    static Stream<Arguments> allCombinations()
    {
        return Stream.of(Arguments.of(null, LocalDate.now(), null), Arguments.of(LocalDate.now(), null, null),
                         Arguments.of(null, null, IllegalArgumentException.class));
    }
}
