package ch20_unittests.ch20_4_parameterized_tests;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.stream.Stream;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import ch20_unittests.basics.Discount;
import ch20_unittests.basics.DiscountCalculatorV4;

/**
 * Unit-Tests zu der Utility-Klasse DiscountCalculatorV4.
 * 
 * @author Michael Inden
 * 
 *         Copyright 2020 by Michael Inden
 */
public class DiscountCalculatorV4Test 
{
	private final DiscountCalculatorV4 calculator = new DiscountCalculatorV4();
	
	@Test
	void testCalcDiscount_NegativeInput_ShouldThrowException() 
	{
		assertThrows(IllegalArgumentException.class, 
		             ()-> calculator.calcDiscount(-1));
	}
	
	@ParameterizedTest(name = "{index}: amount {0} => discount {1}")
	@MethodSource("amountAndDiscount")
	void testCalcDiscount_EquivalenceClasses(int amount, 
	                                         Discount expectedDiscount) 
	{
        Discount result = calculator.calcDiscount(amount);
        
        assertEquals(expectedDiscount, result);
	}

	@ParameterizedTest(name = "{index}: amount {0} => discount {1}")
	@MethodSource("cornerCasesAndDiscount")
	void testCalcDiscount_CornerCases(int amount, 
	                                  Discount expectedDiscount) 
	{
		Discount result = calculator.calcDiscount(amount);
		
        assertEquals(expectedDiscount, result);
	}
	
	private static Stream<Arguments> amountAndDiscount() 
	{
		return Stream.of(Arguments.of(20, Discount.NO_DISCOUNT),
				         Arguments.of(200, Discount.MEDIUM_DISCOUNT),
				         Arguments.of(2000, Discount.BIG_DISCOUNT));
	}
	
	private static Stream<Arguments> cornerCasesAndDiscount() 
	{
		return Stream.of(Arguments.of(49, Discount.NO_DISCOUNT),
				         Arguments.of(50, Discount.MEDIUM_DISCOUNT),
				         Arguments.of(51, Discount.MEDIUM_DISCOUNT),
				         Arguments.of(999, Discount.MEDIUM_DISCOUNT),
				         Arguments.of(1000, Discount.MEDIUM_DISCOUNT), 
				         Arguments.of(1001, Discount.BIG_DISCOUNT));
	}
}
