package ch20_unittests.ch20_4_parameterized_tests;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.time.LocalDate;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ParameterContext;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.converter.ArgumentConversionException;
import org.junit.jupiter.params.converter.ArgumentConverter;
import org.junit.jupiter.params.converter.ConvertWith;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

/**
 * Besipiel fr das Buch "Der Weg zum Java-Profi 5"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class EasierConstructionTest
{

    @Test
    void sundaysBetween()
    {
        final LocalDate start = LocalDate.of(2020, 1, 1);
        final LocalDate end = LocalDate.of(2020, 3, 3);

        var sundays = SundayCalculator.allSundaysBetween(start, end);

        var expectedSundays = List.of(LocalDate.parse("2020-01-05"), LocalDate.parse("2020-01-12"),
                                      LocalDate.parse("2020-01-19"), LocalDate.parse("2020-01-26"),
                                      LocalDate.parse("2020-02-02"), LocalDate.parse("2020-02-09"),
                                      LocalDate.parse("2020-02-16"), LocalDate.parse("2020-02-23"));

        assertEquals(expectedSundays, sundays);
    }

    @Test
    void sundaysBetweenImproved()
    {
        final LocalDate start = LocalDate.of(2020, 1, 1);
        final LocalDate end = LocalDate.of(2020, 3, 1);

        var sundays = SundayCalculator.allSundaysBetween(start, end);

        var dates = List.of("2020-01-05", "2020-01-12", "2020-01-19", "2020-01-26", "2020-02-02", "2020-02-09",
                            "2020-02-16", "2020-02-23");

        var expectedSundays = toListOfLocalDates(dates);

        assertEquals(expectedSundays, sundays);
    }

    private static List<LocalDate> toListOfLocalDates(List<String> dates)
    {
        return dates.stream().map(day -> LocalDate.parse(day)).collect(Collectors.toList());
    }

    @ParameterizedTest(name = "sundays between {0} and {1} => {2}")
    @MethodSource("startAndEndDateAndResults")
    void sundaysBetweenV3(LocalDate start, LocalDate end, List<String> expected)
    {
        var sundays = SundayCalculator.allSundaysBetween(start, end);

        var expectedSundays = toListOfLocalDates(expected);

        assertEquals(expectedSundays, sundays);
    }

    private static Stream<Arguments> startAndEndDateAndResults()
    {
        var args1 = Arguments.of(LocalDate.of(2020, 1, 1), LocalDate.of(2020, 3, 1),
                                 List.of("2020-01-05", "2020-01-12", "2020-01-19", "2020-01-26", "2020-02-02",
                                         "2020-02-09", "2020-02-16", "2020-02-23"));

        var args2 = Arguments.of(LocalDate.of(2020, 4, 1), LocalDate.of(2020, 5, 1),
                                 List.of("2020-04-05", "2020-04-12", "2020-04-19", "2020-04-26"));

        return Stream.of(args1, args2);
    }

    @ParameterizedTest(name = "sundays between {0} and {1} => {2}")
    @MethodSource("startAndEndDateAndResultsV2")
    void sundaysBetweenV3b(LocalDate start, LocalDate end, List<LocalDate> expected)
    {
        var sundays = SundayCalculator.allSundaysBetween(start, end);

        assertEquals(expected, sundays);
    }

    private static Stream<Arguments> startAndEndDateAndResultsV2()
    {
        var args1 = Arguments.of(LocalDate.of(2020, 1, 1), LocalDate.of(2020, 3, 1),
                                 toListOfLocalDates(List.of("2020-01-05", "2020-01-12", "2020-01-19", "2020-01-26",
                                                            "2020-02-02", "2020-02-09", "2020-02-16", "2020-02-23")));

        var args2 = Arguments.of(LocalDate.of(2020, 4, 1), LocalDate.of(2020, 5, 1),
                                 toListOfLocalDates(List.of("2020-04-05", "2020-04-12", "2020-04-19", "2020-04-26")));

        return Stream.of(args1, args2);
    }

    @ParameterizedTest(name = "sundays between {0} and {1} => {2}")
    @MethodSource("startAndEndDateAndResults")
    void sundaysBetweenV4(LocalDate start, LocalDate end,
                          @ConvertWith(ToListOfLocalDatesConverter.class) List<LocalDate> expected)
    {
        final List<LocalDate> result = SundayCalculator.allSundaysBetween(start, end);

        assertEquals(expected, result);
    }

    static class ToListOfLocalDatesConverter implements ArgumentConverter
    {
        @Override
        public Object convert(Object source, ParameterContext context) throws ArgumentConversionException
        {
            final List<String> dates = (List<String>) source;
            return toListOfLocalDates(dates);
        }
    }

    static class SundayCalculator
    {
        public static List<LocalDate> allSundaysBetween(LocalDate start, LocalDate end)
        {
            return List.of();
        }
    }
}
