package ch20_unittests.ch20_4_parameterized_tests;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.time.LocalDate;

import org.junit.jupiter.api.extension.ParameterContext;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.converter.ArgumentConversionException;
import org.junit.jupiter.params.converter.ArgumentConverter;
import org.junit.jupiter.params.converter.ConvertWith;
import org.junit.jupiter.params.provider.CsvSource;

/**
 * Besipiel fr das Buch "Der Weg zum Java-Profi 5"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class SampleDtoConverterTest
{
    @ParameterizedTest
    @CsvSource({ "Michael/1971-02-07/11, Michael, 1971-02-07, 11", "Peter/1973-06-06/9, Peter, 1973-06-06, 9" })
    void testBla(@ConvertWith(SampleDtoConverter.class) SampleDto dto, String expectedInfo, LocalDate expectedDate,
                 int expectedCount)
    {
        assertEquals(expectedInfo, dto.info());
        assertEquals(expectedDate, dto.date());
        assertEquals(expectedCount, dto.count());
    }
}

record SampleDto(String info, LocalDate date, int count)
{
}

class SampleDtoConverter implements ArgumentConverter
{
    @Override
    public Object convert(Object input, ParameterContext parameterContext) throws ArgumentConversionException
    {
        if (input instanceof SampleDto)
            return input;
        // Java 14: instanceof-Verbesserung
        if (input instanceof String strValue)
        {
            // Konvertierung aus String-Repssentation in SampleDto
            try
            {
                final String[] values = strValue.split("/");
                final LocalDate date = LocalDate.parse(values[1].trim());
                final int count = Integer.parseInt(values[2].trim());

                return new SampleDto(values[0].trim(), date, count);
            }
            catch (NumberFormatException | IndexOutOfBoundsException ex)
            {
                String message = input + " is no correct string representation of a SampleDto.";
                throw new ArgumentConversionException(message, ex);
            }
        }
        throw new ArgumentConversionException(input + " is no valid SampleDto");
    }
}