package ch20_unittests.ch20_5_mockito.examples_no_more_interactions;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;

/**
 * Besipiel fr das Buch "Der Weg zum Java-Profi 5"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class MockitoArgCaptorExample
{
    @Test
    public void testGreetingCallCount()
    {
        // Arrange
        final Greeting greeting = mock(Greeting.class);
        when(greeting.greet(anyString())).thenReturn("value");

        // Act
        final Application app = new Application(greeting);
        app.generateMsg("One");
        app.generateMsg("Two");
        app.generateMsg("Three");

        // Assert
        final ArgumentCaptor<String> argumentCaptor = 
                                     ArgumentCaptor.forClass(String.class);
        verify(greeting, times(3)).greet(argumentCaptor.capture());

        final List<String> passedValues = argumentCaptor.getAllValues();
        assertEquals("One", passedValues.get(0));
        assertEquals("Two", passedValues.get(1));
        assertEquals("Three", passedValues.get(2));
    }
}