package ch20_unittests.ch20_6_test_smells;

import static ch20_unittests.ch20_6_test_smells.Tasks.Status.SUCCESS;
import static org.junit.jupiter.api.Assertions.assertAll;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Test;

/**
 * Besipiel fr das Buch "Der Weg zum Java-Profi 5"
 * 
 * @author Michael Inden
 * 
 * Copyright 2020 by Michael Inden 
 */
public class DatabaseConnectorTest
{
    @Test
    void testSmellAssertTrueAndFalse()
    {
        // ARRANGE
        DatabaseConnector db = createDbConnector(10);

        // ACT
        db.process();
        Tasks tasks = db.getTasks();

        // ASSERT
        assertTrue(db.writeCount == 10);
        assertTrue(tasks.totalProcessed == 10);
        assertTrue(tasks.errorCount == 0);
    }

    @Test
    void testSmellAssertTrueAndFalse_Solution()
    {
        // ARRANGE
        DatabaseConnector db = createDbConnector(10);

        // ACT
        db.process();
        Tasks tasks = db.getTasks();

        // ASSERT
        assertEquals(10, db.writeCount);
        assertEquals(10, tasks.totalProcessed);
        assertEquals(0, tasks.errorCount);
    }

    @Test
    void testSmellToManyAsserts()
    {
        // ARRANGE
        DatabaseConnector db = createDbConnector(10);

        // ACT
        db.process();
        Tasks tasks = db.getTasks();

        // ASSERT
        assertEquals(10, db.readCount);
        assertEquals(10, db.writeCount);
        assertEquals(10, db.commitCount);

        assertEquals(10, tasks.totalProcessed);
        assertEquals(0, tasks.errorCount);
        assertEquals(SUCCESS, tasks.status);
    }

    @Test
    void testSmellToManyAsserts_Db_related()
    {
        // ARRANGE
        DatabaseConnector db = createDbConnector(10);

        // ACT
        db.process();

        // ASSERT
        assertEquals(10, db.readCount);
        assertEquals(10, db.writeCount);
        assertEquals(10, db.commitCount);
    }

    @Test
    void testSmellToManyAsserts_Task_related()
    {
        // ARRANGE
        DatabaseConnector db = createDbConnector(10);

        // ACT
        db.process();
        Tasks tasks = db.getTasks();

        // ASSERT
        assertAll(() -> assertEquals(10, tasks.totalProcessed), () -> assertEquals(0, tasks.errorCount),
                  () -> assertEquals(SUCCESS, tasks.status));
    }

    @Test
    void testSmellNoHint()
    {
        final long runtime1 = measure(() -> complexCalculation1());
        final long runtime2 = measure(() -> complexCalculation2());

        assertTrue(runtime2 > runtime1);
    }

    @Test
    void testSmellNoHint_Solution()
    {
        final long runtime1 = measure(() -> complexCalculation1());
        final long runtime2 = measure(() -> complexCalculation2());

        assertTrue(runtime2 > runtime1, "runtime2 " + runtime2 + " ms should be > " + runtime1 + " ms");
    }

    @Test
    void testSmellToStringInAssertEquals()
    {
        final DatabaseConnector dbConnection = createMongoDbConnector();

        assertEquals("mongodb.writeConcern.timeout=10000," + "mongodb.writeConcern.writes=1, " + "mongodb.port=27017, "
                     + "mongodb.password=ksdjsa2455aAYdsj," + "mongodb.user=ABCD", dbConnection.toString());
    }

    @Test
    void testSmellToStringInAssertEquals_Solution()
    {
        final DatabaseConnector dbConnection = createMongoDbConnector();

        assertAll(() -> assertEquals(1, dbConnection.mongodb.writeConcern_writes),
                  () -> assertEquals(27017, dbConnection.mongodb.port),
                  () -> assertEquals("ABCD", dbConnection.mongodb.user));
    }

    @Test
    void testSmellTestingRunningTimes()
    {
        final int count = 1000;

        // $\mbox{\bfseries Laufzeitmessung fr Algorithmus 1 }$
        long runtime1 = 0;

        for (int i = 0; i < count; i++)
        {
            final long start = System.currentTimeMillis();
            performCalculation_V1();
            final long end = System.currentTimeMillis() - start;
            runtime1 += end;
        }

        // $\mbox{\bfseries Laufzeitmessung fr Algorithmus 2}$
        // for-loop mit performCalculation_V1() hier exemplarisch 100 ms
        long runtime2 = 100;

        // $\mbox{\bfseries Fragile Annahme ber die Laufzeiten }$
        assertTrue(runtime2 > runtime1, "calc V2 runtime should be > calc V1 runtime");
    }

    @Test
    void testSmellTestingRunningTimes_Improved()
    {
        final int count = 1000;

        // $\mbox{\bfseries Verbesserte Laufzeitmessung fr Algorithmus 1 }$
        // $\mbox{\bfseries Verbesserung 1: nanoTime() }$
        final long start = System.nanoTime();
        for (int i = 0; i < count; i++)
        {
            performCalculation_V1();
        }
        // $\mbox{\bfseries Verbesserung 2: Gesamtlaufzeit}$
        final long runtime1 = System.nanoTime() - start;

        final long start2 = System.nanoTime();
        // for-loop
        final long runtime2 = System.nanoTime() - start2;

        // $\mbox{\bfseries Immer noch fragile Annahme ber die Laufzeiten }$
        assertTrue(runtime2 > runtime1, "calc V2 runtime should be > calc V1 runtime");
    }

    private void performCalculation_V1()
    {
        // TODO Auto-generated method stub

    }

    private long measure(Runnable runner)
    {

        long start = System.currentTimeMillis();
        runner.run();
        long end = System.currentTimeMillis();

        return end - start;

    }

    private Object complexCalculation1()
    {
        // TODO Auto-generated method stub
        return null;
    }

    private Object complexCalculation2()
    {
        // TODO Auto-generated method stub
        return null;
    }

    // nur fr den Test
    private DatabaseConnector createDbConnector(int writeCount)
    {
        return new DatabaseConnector(writeCount);
    }

    private DatabaseConnector createMongoDbConnector()
    {
        return new DatabaseConnector();
    }
}
